/*
 * watchdog_accessor.cpp
 *
 * Copyright 2019 Sony Imaging Products & Solutions Inc.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 * 
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/platform.h>

/* The watchdog counting clock is a 3 MHz clock. */
#define WDG_CLOCK               (3 * 1000 * 1000)

/* watchdog registers (base address: 0x12080000) */
#define WDG_LOAD                (WDG_REG_BASE + 0x0000)
#define WDG_CONTROL             (WDG_REG_BASE + 0x0008)
#define WDG_INTCLR              (WDG_REG_BASE + 0x000C)
#define WDG_LOCK                (WDG_REG_BASE + 0x0C00)

/* value of WDG_CONTROL */
#define WDG_CONTROL_ENABLE      0x00000003
#define WDG_CONTROL_DISABLE     0x00000000

/* value of WDG_INTCLR */
#define WDG_INTCLR_CLEAR        0x00000000

/* value of WDG_LOCK */
#define WDG_LOCK_UNLOCK         0x1ACCE551
#define WDG_LOCK_LOCK           0x00000000

void enable_watchdog()
{
    writel(WDG_LOCK_UNLOCK, WDG_LOCK);
    writel(WDG_CONTROL_DISABLE, WDG_CONTROL);
    writel(WDG_INTCLR_CLEAR, WDG_INTCLR);

    u64 value = (CONFIG_WATCH_DOG_SECONDS * WDG_CLOCK) / 2;
    if (value > (u32)(~0x0)) {
        value = (u32)(~0x0);
    }
    writel((u32)(value), WDG_LOAD);

    writel(WDG_CONTROL_ENABLE, WDG_CONTROL);
    writel(WDG_LOCK_LOCK, WDG_LOCK);
}
