/**
 * Copyright (c) 2009-2014 HiSilicon Technologies Co., Ltd.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <platform.h>
//#include "config.h"
#ifdef CONFIG_DDR_TRAINING_V2
#include <ddr_interface.h>
#endif

#define HI_SYS_CTL_REG	0x12020000

#define DDR0_BASE_REG	0x12068000
#define DDR0_PLL_REG    0x1206c000

static inline void delay(unsigned int num)
{
	volatile unsigned int i;

	for (i = 0; i < (100 * num); i++)
		__asm__ __volatile__("nop");
}


static inline void DWB(void) /* drain write buffer */
{
}

static inline unsigned int readl(unsigned addr)
{
	unsigned int val;

	val = (*(volatile unsigned int *)(addr));
	return val;
}

static inline void writel(unsigned val, unsigned addr)
{
	DWB();
	(*(volatile unsigned *) (addr)) = (val);
	DWB();
}

void reset_cpu(unsigned long addr)
{
	/* Any value to this reg will reset the cpu */
	writel(0x12345678, (REG_BASE_SCTL + REG_SC_SYSRES));
	delay(5000);
}

#define REG_SC_GEN32 0xa0
#define REG_SC_GEN33 0xa4
#define REG_DDR_DMC0_CFG_DDRMODE (DDR0_BASE_REG + 0x50)
#define REG_DDR_DMC0_CFG_TIMING2 (DDR0_BASE_REG + 0x108)
#define REG_DDR_PHY0_TRAINCTRL   (DDR0_PLL_REG + 0x48)
#define REG_DDR_PHY0_INITCTRL    (DDR0_PLL_REG + 0x4)
#define REG_DDR_DMC0_CTRL_SREF   DDR0_BASE_REG
#define REG_DDR_DMC0_IN_SREF     (DDR0_BASE_REG + 0x294)

void ddr_set_sref(unsigned int val)
{
	int cnt = 0;

	writel(val, REG_DDR_DMC0_CTRL_SREF);
	while ((!(readl(REG_DDR_DMC0_IN_SREF) & 0x1)) && (cnt < 10000)) {
		cnt++;
	}

	if (cnt >= 10000) {
		reset_cpu(0);
	}
}


void hi3519_ddr_init(void)
{
	unsigned int initctrl_rank0, initctrl_rank1, rank_num;

	/* Read rank0 HW DDR training value */
	initctrl_rank0 = readl(REG_BASE_SCTL + REG_SC_GEN32);
	/* Read rank1 HW DDR training value */
	initctrl_rank1 = readl(REG_BASE_SCTL + REG_SC_GEN33);
	/* 0: one rank, 1: two rank */
	rank_num = (readl(REG_DDR_DMC0_CFG_DDRMODE) >> 20) & 0x3;

	if (initctrl_rank0 || (initctrl_rank1 && rank_num)) {
		unsigned int timing;
		unsigned int timing_backup;
		unsigned int rank_ctrl;

		/* Save the Auto-self refresh timing */
		timing = readl(REG_DDR_DMC0_CFG_TIMING2);
		timing_backup = timing & 0x7FF;

		/* Set Auto-self refresh timing to 0 */
		writel((timing & 0xFFFFF800), REG_DDR_DMC0_CFG_TIMING2);

		rank_ctrl = readl(REG_DDR_PHY0_TRAINCTRL);
		if (initctrl_rank0) {
			/* set bit[0:3] = 0 to training rank0 */
			writel(rank_ctrl & 0xfffffff0, REG_DDR_PHY0_TRAINCTRL);
			/* Start HW DDR training */
			writel(initctrl_rank0, REG_DDR_PHY0_INITCTRL);
			/* wait HW DDR training finish */
			do {
				initctrl_rank0 = readl(REG_DDR_PHY0_INITCTRL) & 0x7FFFF;
			} while(initctrl_rank0);
		}

		if (initctrl_rank1 && rank_num) {
			/* set bit[0:3] = 1 to training rank1 */
			writel((rank_ctrl & 0xfffffff0) | 0x1, REG_DDR_PHY0_TRAINCTRL);
			/* Start HW DDR training */
			writel(initctrl_rank1, REG_DDR_PHY0_INITCTRL);
			/* wait HW DDR training finish */
			do {
				initctrl_rank1 = readl(REG_DDR_PHY0_INITCTRL) & 0x7FFFF;
			} while(initctrl_rank1);
		}
		writel(rank_ctrl, REG_DDR_PHY0_TRAINCTRL); /* restore */

		/* Exit Auto-self refresh */
		ddr_set_sref(0x2);

		/* Restore the Auto-self refresh timing */
		timing = readl(REG_DDR_DMC0_CFG_TIMING2);
		timing = (timing & 0xFFFFF800) | timing_backup;
		writel(timing, REG_DDR_DMC0_CFG_TIMING2);
	}

	/* Clear fifo */
	initctrl_rank0 = readl(REG_DDR_PHY0_INITCTRL);
	initctrl_rank0 = initctrl_rank0 | (1<<15);
	writel(initctrl_rank0, REG_DDR_PHY0_INITCTRL);
	initctrl_rank0 = readl(REG_DDR_PHY0_INITCTRL);
	initctrl_rank0 = initctrl_rank0 & ~(1<<15);
	writel(initctrl_rank0, REG_DDR_PHY0_INITCTRL);
}

#ifdef CONFIG_SVB_ENABLE

#define HI_PMC_CTL_REG		0x120a0000
#define HPM_CORE_VOL_REG	(HI_PMC_CTL_REG + 0x0)
#define HPM_CPU_VOL_REG		(HI_PMC_CTL_REG + 0x4)
#define HPM_DDR_VOL_REG		(HI_PMC_CTL_REG + 0x8)
#define HPM_MDA_VOL_REG		(HI_PMC_CTL_REG + 0xc)
#define CYCLE_NUM		4
#define HPM_CORE_REG0	0x2c
#define HPM_CORE_REG1	0x30
#define HPM_CPU_REG0	0x3c
#define HPM_CPU_REG1	0x40
#define HPM_DDR_REG0	0x4c
#define HPM_DDR_REG1	0x50
#define HPM_MDA_REG0	0x5c
#define HPM_MDA_REG1	0x60
#define PERI_CRG_PLL1	0x4
#define PERI_CRG13	0x34
#define CPU_A17_CKSEL_MASK	0x70

#define HPM_POWER_PROFILE_REG	0x1202015c
#define FOUR_WAY_POWER	0
#define THREE_WAY_POWER	1
#define TWO_WAY_POWER	2

#define OTP_CORE_HPM_IDDQ	0x12032000
#define OTP_MDA_HPM_IDDQ	0x12032004

static unsigned hpm_value_avg(unsigned int *val)
{
	unsigned int i;
	unsigned tmp = 0;

	for (i = 0; i < 4; i++)
		tmp += val[i] >> 2;

	return tmp >> 2;
}

static void get_hpm_value(unsigned int *hpm_core, unsigned int *hpm_cpu,
		unsigned int *hpm_ddr, unsigned int *hpm_mda)
{
	int i;
	unsigned int temp;
	unsigned int core_value[4] = {0, 0, 0, 0};
	unsigned int mda_value[4] = {0, 0, 0, 0};

	for (i = 0; i < CYCLE_NUM; i++) {
		delay(100);

		/* core */
		temp = readl(HI_PMC_CTL_REG + HPM_CORE_REG0);
		core_value[1] += (temp >> 12) & 0x3ff;
		core_value[0] += temp & 0x3ff;
		temp = readl(HI_PMC_CTL_REG + HPM_CORE_REG1);
		core_value[3] += (temp >> 12) & 0x3ff;
		core_value[2] += temp & 0x3ff;

		/* mda */
		temp = readl(HI_PMC_CTL_REG + HPM_MDA_REG0);
		mda_value[1] += (temp >> 12) & 0x3ff;
		mda_value[0] += temp & 0x3ff;
		temp = readl(HI_PMC_CTL_REG + HPM_MDA_REG1);
		mda_value[3] += (temp >> 12) & 0x3ff;
		mda_value[2] += temp & 0x3ff;
	}

	*hpm_core = hpm_value_avg(core_value);
	*hpm_mda = hpm_value_avg(mda_value);
}

static void start_hpm(unsigned int *hpm_core, unsigned int *hpm_cpu,
		unsigned int *hpm_ddr, unsigned int *hpm_mda)
{
	writel(0x05000003, HI_PMC_CTL_REG + 0x28);
	writel(0x05000003, HI_PMC_CTL_REG + 0x58);

	delay(500);

	get_hpm_value(hpm_core, hpm_cpu, hpm_ddr, hpm_mda);
}

static void set_hpm_core_volt(unsigned int hpm_core_value)
{
	unsigned int otp_core_hpm_iddq;

	otp_core_hpm_iddq = (readl(OTP_CORE_HPM_IDDQ) >> 16) & 0x3f;

	if (hpm_core_value > 270) {
		if (otp_core_hpm_iddq > 9 || !otp_core_hpm_iddq)
			writel(0x3500c7, HPM_CORE_VOL_REG);/*0.89v*/
		else
			writel(0x2900c7, HPM_CORE_VOL_REG);/*0.91v*/

	} else if (hpm_core_value > 245)
		writel(0x2900c7, HPM_CORE_VOL_REG);/*0.91v*/
	else if (hpm_core_value > 220)
		writel(0x1700c7, HPM_CORE_VOL_REG);/*0.94v*/
	else
		writel(0x0b00c7, HPM_CORE_VOL_REG);/*0.96v*/
}
static void set_hpm_mda_volt(unsigned int hpm_mda_value)
{
	unsigned int otp_mda_hpm_iddq;

	otp_mda_hpm_iddq =  (readl(OTP_MDA_HPM_IDDQ) >> 16) & 0x3f;

	if (hpm_mda_value > 245) {
		if (otp_mda_hpm_iddq > 13 || !otp_mda_hpm_iddq)
			writel(0x4600c7, HPM_MDA_VOL_REG);/*0.86v*/
		else
			writel(0x3a00c7, HPM_MDA_VOL_REG);/*0.88v*/
	} else if (hpm_mda_value > 220)
		writel(0x3a00c7, HPM_MDA_VOL_REG);/*0.88v*/
	else if (hpm_mda_value > 195)
		writel(0x2200c7, HPM_MDA_VOL_REG);/*0.92v*/
	else
		writel(0x1000c7, HPM_MDA_VOL_REG);/*0.95v*/
}

static void start_svb(void)
{
	unsigned int hpm_core;
	unsigned int hpm_cpu;
	unsigned int hpm_ddr;
	unsigned int hpm_mda;
	unsigned int otp_hpm_core;
	unsigned int otp_hpm_mda;

	/*set start voltage, CORE/MDA0 is 0.9v*/
	writel(0x2f00c7, HPM_CORE_VOL_REG);
	writel(0x2e00c7, HPM_MDA_VOL_REG);


	start_hpm(&hpm_core, &hpm_cpu, &hpm_ddr, &hpm_mda);

	/* modify hpm value according to otp hpm value */
	otp_hpm_core = readl(OTP_CORE_HPM_IDDQ) & 0x3ff;
	if (otp_hpm_core && otp_hpm_core < hpm_core)
		hpm_core = otp_hpm_core;

	otp_hpm_mda = readl(OTP_MDA_HPM_IDDQ) & 0x3ff;
	if (otp_hpm_mda && (otp_hpm_mda + 15) < hpm_mda)
		hpm_mda = otp_hpm_mda + 15;

	set_hpm_core_volt(hpm_core);
	set_hpm_mda_volt(hpm_mda);
}
#endif

void start_ddr_training(unsigned int base)
{
#ifdef CONFIG_SVB_ENABLE
	start_svb();
#endif

	hi3519_ddr_init();

#ifdef CONFIG_DDR_TRAINING_V2
	int ret = 0;

	/* ddr training function */
	ret = ddr_sw_training_if(0);
	if (ret)
		reset_cpu(0);
#endif
	/*the value should config after trainning, or
	  it will cause chip compatibility problems*/
	writel(0x401, DDR0_BASE_REG + 0x28);
}
