#include <platform.h>


/*-----------------------------------------------------------------
  void cpu_init_crit()
------------------------------------------------------------------ */ 
.globl cpu_init_crit
cpu_init_crit:   
    /*
    * disable MMU stuff and caches
    */
    mrc     p15, 0, r1, c1, c0, 0      /* get control register (c1)*/
    bic     r1, r1, #0x00001800        /*-IZ-*/
    bic     r1, r1, #0x00000007        /*-CAM*/
    mcr     p15, 0, r1, c1, c0, 0
    bx      lr
/*-----------------------------------------------------------------
  void open_I_cache()
------------------------------------------------------------------ */ 
.globl open_I_cache
open_I_cache:
    push    {lr}
    
    /*
     * A7: you must set SMP bit ACTLR register before
     * enable I cache and D cache
     */
    mrc     p15, 0, r0, c1, c0, 1
    orr     r0, #(1 << 6)
    mcr     p15, 0, r0, c1, c0, 1
    
    bl      invalidate_icache
    
    mrc     p15, 0, r1, c1, c0, 0      /* get control register (c1)*/
    orr     r1, r1, #(1 << 12)         /* I bit (Instruction cache)*/ 
    mcr     p15, 0, r1, c1, c0, 0
    isb
    pop     {pc}


/*-----------------------------------------------------------------
  void close_I_cache()
------------------------------------------------------------------ */     
.globl close_I_cache
close_I_cache:
    mrc     p15, 0, r1, c1, c0, 0      /* get control register (c1)*/
    bic     r1, r1, #(1 << 12)         /* I bit (Instruction cache)*/ 
    mcr     p15, 0, r1, c1, c0, 0
    b       invalidate_icache

/*-----------------------------------------------------------------
  void clean_invalidate_dcache(u32 clean)
------------------------------------------------------------------ */ 
clean_invalidate_dcache:
    mov     r3, #0
    mcr     p15, 2, r3, c0, c0, 0           /* select L1 data cache */
    isb                                     /* isb to sych the new cssr&csidr */
    mrc     p15, 1, r3, c0, c0, 0           /* Read Current Cache Size Identification Register */
    ldr     r1, =0x7fff
    and     r3, r1, r3, LSR #13             /* r3 = number of sets in cache */
    mov     ip, #0
1:
    mov     r1, #0                          /* r1->set counter */
2:
    mov     r2, ip, LSL #30
    orr     r2, r1, LSL #5                  /* r2->set/way cache-op format */
    cmp     r0, #0
    mcrne   p15, 0, r2, c7, c14, 2          /* Clean and Invalidate line described by r2 */
    mcreq   p15, 0, r2, c7, c6, 2           /* Invalidate line described by r2 */
    add     r1, r1, #1                      /* Increment set counter */

    cmp     r1, r3                          /* Check if the last set is reached... */
    ble     2b                              /* if not, continue the set_loop */
    add     ip, ip, #1                      /* else, Increment way counter */

    cmp     ip, #4                          /* Check if the last way is reached... */
    blt     1b                              /* if not, continue the way_loop */

    bx      lr

invalidate_icache:
    mov     r0, #0                  @ set up for MCR
    mcr     p15, 0, r0, c8, c7, 0   @ invalidate TLBs
    mcr     p15, 0, r0, c7, c5, 0   @ invalidate icache
    bx      lr

