#ifndef __LIB_H__
#define __LIB_H__

#include <types.h>

void*  memset(void * s,int c,size_t count);
void*  memcpy(void * dest,const void *src,size_t count);
int    memcmp(const void * cs,const void *ct,size_t count);

size_t strlen(const char * s);
char*  strcpy(char * dest,const char *src);

#ifdef CFG_DEBUG
size_t strnlen(const char * s, size_t count);
void printf (const char *fmt, ...);
void set_time_mark();
unsigned long get_time_ms();

#define PRINTDUMP(buf, count) do {          \
    int i; for(i = 0; i < count; ++i) {     \
        if (!(i % 0xf)) printf("\n");       \
        printf("%x ",buf[i]);               \
    } printf("\n");                         \
}while(0)

#else /*CFG_DEBUG*/

#define printf(...)
#define PRINTDUMP(...)
#define set_time_mark(...)
#define get_time_ms(...)

#endif /*CFG_DEBUG*/


int malloc_init(void);
void *malloc(size_t size);
void free(void *mem_ptr);

void udelay(unsigned long us);

void debug_info (char * printbuffer,char value);

#undef offsetof
#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)

#undef reg_get
#define reg_get(addr) (*(volatile unsigned int *)(addr))

#undef reg_set
#define reg_set(addr, val) (*(volatile unsigned int *)(addr) = (val))

/*big endian*/
#define read_u32_big(dst, src) do { \
    (dst) =                         \
    (*((u8*)(src)    ) << 24) |     \
    (*((u8*)(src) + 1) << 16) |     \
    (*((u8*)(src) + 2) << 8)  |     \
    (*((u8*)(src) + 3));            \
    (src) += 4;                     \
}while(0)

/*big endian*/
#define read_u16_big(dst, src) do { \
    (dst) =                         \
    (*((u8*)(src)    ) << 8)  |     \
    (*((u8*)(src) + 1));            \
    (src) += 2;                     \
}while(0)
    
#ifndef readl
#define readl(addr) (*(volatile unsigned int *)(addr))
#endif

#ifndef writel
#define writel(val, addr) (*(volatile unsigned int *)(addr) = (val))
#endif

#define max(a,b) (((a) > (b)) ? (a) : (b))
#define min(a,b) (((a) < (b)) ? (a) : (b)) 

#endif /*__LIB_H__*/

