#ifndef __PLATFORM_HISILICON_H__
#define __PLATFORM_HISILICON_H__

/******************************************************************************
* 
*               ON CHIP RAM
*               -------------------------
*               | heap                  |
*               +-----------------------+
*               | stack                 |
*               +-----------------------+
*               | global variable       |
*               +-----------------------+
*               | page table            |
* RAM_BASE--->  -------------------------
* 
* 
*               ON CHIP ROM
*               -------------------------
*               | rodata                |
*               +-----------------------+
*               | text                  |
* ROM_BASE--->  -------------------------
* 
* 
* 
******************************************************************************/
#define CONFIG_DDR_TRAINING_V2

#undef reg_get
#define reg_get(addr) (*(volatile unsigned int *)(addr))

#undef reg_set
#define reg_set(addr, val) (*(volatile unsigned int *)(addr) = (val))

#define RAM_SIZE        0x7000  /* 28k */

#define STACK_SIZE      0x500
#define STACK_ADDR      (CFG_RAM_BASE_ADDR + STACK_SIZE)

#define HEAP_ADDR       STACK_ADDR
#define CP_STEP1_ADDR   HEAP_ADDR
#define JP_ADDR		HEAP_ADDR    /* ram imgae offset(1.25k) :0x04010500 */
#define CP_STEP1_SIZE        0x4f00  /* copy 19.75KB for the first time */

#define CONFIG_ENV_SIZE            0x40000    /*include ENV_HEADER_SIZE */
#define CONFIG_ENV_SECT_SIZE CONFIG_ENV_SIZE
#define CONFIG_SYS_MALLOC_LEN      (CONFIG_ENV_SIZE + 128*1024)
/* size in bytes reserved for initial data */
#define CONFIG_SYS_GBL_DATA_SIZE   128

/*---------------------------------------------------------------------------
 * misc register
 *---------------------------------------------------------------------------*/
#define REG_BASE_MISC	0x12030000
#define MISC_CTRL62_OFST	0xf8
#define MISC_CTRL63_OFST	0xfc
#define MISC_CTRL64_OFST	0x100

/*-----------------------------------------------------------------
 * sysctrl register 
------------------------------------------------------------------ */ 
#define REG_BASE_SCTL           0x12020000
#define REG_SC_CTRL             0x0000
#define TIMEREN0_SEL		(1<<16)
#define MODE_SEL		0x3
#define MODE_STATUS		0xf
#define MODE_MASK		0x7
#define MODE_SLOW		0x2
#define MODE_NORMAL		0x4
#define REG_SC_SYSRES           0x0004
#define REG_SC_PLLFCTRL         0x0014
#define REG_SC_LOCKEN           0x0044
#define SC_UNLOCK_VAL           0x1ACCE551
#define REG_SC_SYSSTAT          0x008c
#define BOOTROM_SEL_IN		(1<<31)	
#define BIT_BOOT_MODE		4
#define BIT_DDR_CA_EN		13
#define BIT_SECURE_BOOT		14
#define BIT_UPDATE_FROM_SDIO0	23

#define REG_SC_GEN2             0x0138
#define REG_SC_GEN4             0x0140
#define REG_SC_GEN5             0x0144
#define REG_SC_GEN6             0x0148

/*-----------------------------------------------------------------
 * CRG register 
------------------------------------------------------------------ */ 
#define REG_BASE_CRG            0x12010000

#define REG_PERI_CRG0		0x0
#define REG_PERI_CRG1		0x0004
#define REG_PERI_CRG4		0x0010
#define REG_PERI_CRG5		0x0014
#define REG_PERI_CRG8		0x0020
#define REG_PERI_CRG9		0x0024

#define REG_PERI_CRG13		0x0034

#define REG_PERI_CRG57		0x00E4
#define REG_PERI_CRG58		0x00E8
#define REG_PERI_CRG67		0x010C
#define REG_PERI_CRG79		0x013C

#define APLL_LOCK		0x1
#define DPLL_LOCK		(0x1 << 2)
#define VPLL_LOCK		(0x1 << 3)
#define EPLL_LOCK		(0x1 << 4)
#define PLL_LOCK		(APLL_LOCK | DPLL_LOCK | VPLL_LOCK | EPLL_LOCK)

#define SERIAL_CLOCK_24M		(0x1<<25)

/*-----------------------------------------------------------------
 *serial base address and clock 
------------------------------------------------------------------ */ 
#define REG_BASE_SERIAL0	0x12100000
#define REG_BASE_SERIAL1	0x12101000

#define CONFIG_PL011_CLOCK      24000000  /* Hi3519 Serial need 24M clock input */

#define CONFIG_CONS_INDEX       0 /* select the default console */


/*-----------------------------------------------------------------
 * timer0 register 
------------------------------------------------------------------ */ 
#define REG_BASE_TIMER0         0x12000000
#define TIMER_LOAD	        0x0000
#define TIMER_VALUE             0x0004
#define TIMER_CONTROL		0x0008
#define TIMER_EN     (1<<7)
#define TIMER_MODE   (1<<6)
#define TIMER_PRE    (0<<2)
#define TIMER_SIZE   (1<<1)

#define CFG_TIMERBASE           REG_BASE_TIMER0
#define READ_TIMER              (*(volatile unsigned long *)\
		                (CFG_TIMERBASE + TIMER_VALUE))
#define TIMER_FEQ 		(3000000)
/* how many ticks per second. show the precision of timer. */
#define TIMER_DIV		(1)
#define CONFIG_SYS_HZ           (TIMER_FEQ/TIMER_DIV)
/*-----------------------------------------------------------------
 * boot   Configuration
------------------------------------------------------------------ */ 
#define BOOT_SEL_MASK       (0x3)
#define BOOT_SEL_SPI_FLASH  (0)
#define BOOT_SEL_NAND       (1)
#define BOOT_SEL_EMMC       (2)

#define UPDATE_FROM_SDIO0_MASK	(0x1)
#define SECURE_BOOT_MASK	(0x1)
#define DDR_SCRAMBLING_MASK	(0x1)

#define BOOTROM_STAT_STACK       0x0
#define BOOTROM_STAT_START       0x1
#define BOOTROM_STAT_MAIN        0x2
#define BOOTROM_STAT_SLOW        0x3
#define BOOTROM_STAT_NORMAL      0x4
#define BOOTROM_STAT_SELFBOOT    0x5
#define BOOTROM_STAT_UART        0x6
#define BOOTROM_STAT_SPI         0x7
#define BOOTROM_STAT_SPI_NAND    0x8
#define BOOTROM_STAT_NAND        0x9
#define BOOTROM_STAT_BOOT        0xa
#define BOOTROM_STAT_ERR         0xb
#define BOOTROM_STAT_INTR        0xc
#define BOOTROM_STAT_EMMC        0xd
#define BOOTROM_STAT_SDIO	 0xe

#define SPI_BASE_ADDR          0x14000000
#define DDR_BASE_ADDR          0x40000000

#define SRAM_DOWNLOAD_ADDR	HEAP_ADDR
#define DDR_DOWNLOAD_ADDR	0x81000000

#define SRAM_DOWNLOAD_SIZE	0x4f00		/* 19.75k for selfboot */
#define DDR_DOWNLOAD_SIZE	0x80000		/* 512K */

/* compute the size of secure boot image */
#define CAL_SEC_IMG_SIZE(n_len, ddr_init_len, uboot_len)\
	(2048 + 2 * sizeof(unsigned int) + 2 * (n_len)\
	 + (ddr_init_len) + (uboot_len))

#define CAL_UBOOT_DDR_START_ADDR(ddr_init_len, n_len)\
	(DDR_DOWNLOAD_ADDR + 2048 + 2 * sizeof(unsigned int)\
	 + (ddr_init_len) + (n_len))

#define SELF_BOOT_START_MAGIC     (0x7a696a75)
#define SELF_BOOT_DOWNLOAD_MAGIC  (0x444f574e)

/* --------------------------------------------------------------------
 * DDR Training
 * -------------------------------------------------------------------- */
#define STACK_TRAINING			0x04017ff0
#define _HI3519_V101                  (0x003519101LL)
#define _HI3519_MASK                  (0xFFFFFFFFFFLL)
#define REG_SC_DDRT0			0x0090
#define NORMAL_BOOTMODE_OFFSET      9
#define NORMAL_BOOTMODE_MASK        3

/* sram star addr(0x04010500) + ddr_init.bin length size(0x4) + *
 * ras_pub_key.bin size(0x800) + rsa n and e param size(0x8) */
#define RAM_START_ADRS                  (0x04010500 + 0x800 + 0x8 + 0x4)

#define SYS_CTRL_REG_BASE               0x12020000

/***********************************************************************************
 * EMMC
 ***********************************************************************************/
#define REG_IO_BASE		0x12040000
#define SDIO_MUXCTRL_REG_START	0xf8
#define SDIO_MUXCTRL_REG_END	0x118

#define REG_CRG_BASE		REG_BASE_CRG
#define REG_BASE_SDIO0		0x100C0000
#define REG_BASE_SDIO1		0x100D0000
#define REG_BASE_EMMC		0x100E0000
#define PERI_CRG_SDIO_0_1	0xc4

#define PADCTRL_REG_BASE	0x12040800
#define PADCTRL_REG51		0x0CC
#define PADCTRL_REG52		0x0D0
#define PADCTRL_REG55		0x0DC

/*************************************************************
 *		Secure boot
 *************************************************************/
#if 0
enum secure_boot_type {
	SECURE_BOOT_TYPE_UART = 0x73616665, /* safe */
	SECURE_BOOT_TYPE_NORMAL,
	SECURE_BOOT_TYPE_SDIO
};

typedef struct data_info {
	unsigned int rsa_n_len;		/* length of RSA n parameter */
	unsigned int rsa_e1_len;	/* length of RSA e1 parameter */
	unsigned int key_addr;		/* start addr of key.bin */
	unsigned int ddr_init_len;	/* lenght of ddr_init.bin */
	unsigned int ddr_init_addr;	/* start addr of ddr_init.bin */
	unsigned int ddr_init_sig_addr;	/* start addr of ddr_init_signatrue.bin */
	unsigned int ddr_init_sig_len;	/* length of ddr_init_signature.bin */
	unsigned int uboot_len;		/* length of u-boot.bin*/
} boot_data_info_s;
#endif

#define RSA_E1_PARA_OFST	(0x4)
#define KEY_ADDR_OFST		(0x8)
#define DDR_INIT_LEN_ADDR_OFST	(0x800)
#define DDR_INIT_ADDR_OFST	(DDR_INIT_LEN_ADDR_OFST + 0x4)

#define KEY_BUF_SIZE		(0x800)
#define SHA_DATA_SIZE		(0x20)

/*
 *AES CBC decryptiong, use key 0, which is configured by cpu,
 * key length is 128 bit.
 */
#define CIPHER_CTRL_SEL		((0x2 << 4) | (0x1 << 1) | 0x1)

/*------------------------------------------------------------------------
 *misc register
 *------------------------------------------------------------------------*/
#define REG_BASE_MISC		0x12030000
#define ROOTKEY_AES_0_OFST	0x3000
#define ROOTKEY_AES_1_OFST	0x3004
#define ROOTKEY_AES_2_OFST	0x3008
#define ROOTKEY_AES_3_OFST	0x300C

/*-------------------------------------------------------------------------
 * cipher register
 *-------------------------------------------------------------------------*/
#define REG_BASE_CIPHER		0x10080000
#define CIPHER_KEY_0_OFST	0x90
#define CIPHER_KEY_1_OFST	0x94
#define CIPHER_KEY_2_OFST	0x98
#define CIPHER_KEY_3_OFST	0x9c
#define CHAN1_IBUF_NUM_OFST		(0x1000 + 128)
#define CHAN1_IBUF_CNT_OFST		(0x1000 + 128 + 0x4)
#define CHAN1_CIPHER_CTRL_OFST		(0x1000 + 128 + 0x10)
#define CHAN1_SRC_LST_SADDR_OFST	(0x1000 + 128 + 0x14)
#define CHAN1_OBUF_NUM_OFST		(0x1000 + 128 + 0x3c)
#define CHAN1_DEST_LST_SADDR_OFST	(0x1000 + 128 + 0x4c)
#define INT_RAW_OFST			0x1408

#define CHAN1_OUT_INTR_MASK		(0x1 << 9)
#define CHAN1_INPUT_BUF_NUM		(0x1)
#define CHAN1_OUTPUT_BUF_NUM		(0x1)

/*-------------------------------------------------------------------------------
 * hash register
 *-------------------------------------------------------------------------------*/
#define REG_BASE_HASH		0x10090000
#define TOTAL_LEN1_OFST		0x0
#define TOTAL_LEN2_OFST		0x4
#define HASH_STATUS_OFST	0x8
#define HASH_CTRL_OFST		0xc
#define SHA_START_OFST		0x10
#define DMA_START_ADDR_OFST	0x14
#define DMA_LEN_OFST		0x18
#define SHA_OUT1_OFST		0x30
#define SHA_OUT2_OFST		0x34
#define SHA_OUT3_OFST		0x38
#define SHA_OUT4_OFST		0x3c
#define SHA_OUT5_OFST		0x40
#define SHA_OUT6_OFST		0x44
#define SHA_OUT7_OFST		0x48
#define SHA_OUT8_OFST		0x4c

#define HASH_READY_MASK		(0x1)
#define SHA_START_EN		(0x1)
#define REC_READY_MASK		(0x1 << 3)

/* read message by DMA, SHA256 mode, mcu key
 * big end, disable sha init update */
#define HASH_CTRL_VAL		((0x1 << 4) | (0x1 << 1))

/*--------------------------------------------------------------------------------
 * rsa register
 *--------------------------------------------------------------------------------*/
#define REG_BASE_RSA			0x120b0000
#define RSA_BUSY_REG_OFST		0x50
#define RSA_MODE_REG_OFST		0x54
#define RSA_WSEC_REG_OFST		0x58
#define RSA_WPKT_REG_OFST		0x5c
#define RSA_RRSLT_REG_OFST		0x64
#define RSA_START_REG_OFST		0x68

#define RSA_BUSY_MASK			(0x1)
#define RSA_START_EN			(0x1)

#define RSA_KEY_LEAST_WIDTH		128
#define RSA_KEY_WIDTH_SEL(x)		(((x) / RSA_KEY_LEAST_WIDTH) - 1)

/*-----------------------------------------------------------------------------------
 * rng_gen register 
 *-----------------------------------------------------------------------------------*/
#define REG_BASE_RNG_GEN		0x120C0000
#define TRNG_DSTA_FIFO_DATA_OFST	0x204
#define TRNG_DATA_ST_OFST		0x208

#define BIT_TRNG_FIFO_DATA_CNT		0x8
#define TRNG_FIFO_DATA_CNT_MASK		0xff

#define SECURE_BOOT_OVER_TIME		500	/* 500 ms */	

#if 0
/* list head for aes decryption */
typedef struct buf_list_head {
	unsigned int addr;			/* start addr for data buffer */
	union {
		unsigned int buf_len :20;	/* buffer length */
		unsigned int rsv1    :1;
		unsigned int iv_set  :1;	/* update iv flag */
		unsigned int last_lst:1;	/* last list flag */
		unsigned int rsv2    :9;

		unsigned int data_len:20;	/* data length */
		unsigned int davs    :12;

		unsigned int val[2];
	} ctrl_data;

	unsigned int iv_start_addr;		/* start addr of iv */
} buf_list_head_s;
#endif

/* serial driver */
#define CONFIG_PL011_SERIAL 1
#define UART0_REG_BASE                  0x12100000
#define CONFIG_PL01x_PORTS      \
{(void *)UART0_REG_BASE, (void *)UART1_REG_BASE, \
	(void *)UART2_REG_BASE, (void *)UART3_REG_BASE}
#define CONFIG_CONS_INDEX       0

#define CFG_SERIAL0                   UART0_REG_BASE
#define CONFIG_CUR_UART_BASE          CFG_SERIAL0

/****************************************************/
#define CFG_TIMER_CLK (3000000)
#define TIMER0_REG_BASE                 0x12000000

#define REG_TIMER_RELOAD                0x0
#define REG_TIMER_VALUE                 0x4
#define REG_TIMER_CONTROL               0x8
#define CFG_TIMER_CTRL          0xC2

#define CONFIG_SVB_ENABLE

#include "serial_reg.h"

#endif  /*__PLATFORM_HISILICON_H__*/
