#!/bin/bash -eux

progname="$0"
workdir=${PWD}
dirname="${workdir}/`dirname $progname`"

usage()
{
   cat << END
Usage: $progname [options]
Example: $progname

Options:
        -h    Show help
        -r    enable release build

END
}

target="hi3519v101"
CROSS_COMPILE="arm-hisiv600-linux-"
RELEASE_BUILD=
while getopts crh opt "$@"
do
    case $opt in
        c) CROSS_COMPILE="arm-sony-linux-gnueabi-armv7a-dev-";;
        r) RELEASE_BUILD="yes";;
        h) usage; exit 0;;
        *) usage; exit 1;;
    esac
done
shift $(($OPTIND - 1))

DTC=`which dtc`
if [ $? -ne 0 ]; then
    echo "not found device tree compiler. abort."
    exit 1
fi

DISTCC_PARALLEL_NUM=`cat /proc/cpuinfo | grep processor | wc -l`

MAKE="make"
ARCH="arm"

which "${CROSS_COMPILE}gcc" 2>&1 > /dev/null
ret=$?
if [ ${ret} -ne 0 ]; then
    echo "not found cross compiler"
    exit ${ret}
fi

${MAKE} ARCH=${ARCH} CROSS_COMPILE=${CROSS_COMPILE} clean
${MAKE} ARCH=${ARCH} CROSS_COMPILE=${CROSS_COMPILE} RELEASE_BUILD=${RELEASE_BUILD} hi3519v101_config
${MAKE} ARCH=${ARCH} CROSS_COMPILE=${CROSS_COMPILE} RELEASE_BUILD=${RELEASE_BUILD} -j${DISTCC_PARALLEL_NUM}
${MAKE} ARCH=${ARCH} CROSS_COMPILE=${CROSS_COMPILE} env

pushd ddr_init
make
popd

exit 0
