/*
* Copyright (c) 2005-2014 HiSilicon Technologies Co., Ltd.
*
* This program is free software; you can redistribute  it and/or modify it
* under  the terms of  the GNU General Public License as published by the
* Free Software Foundation;  either version 2 of the  License, or (at your
* option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*/
#ifndef __DRV_CIPHER_INTF_H__
#define __DRV_CIPHER_INTF_H__

#include <common.h>
#include "drv_cipher_define.h"

#ifdef __cplusplus
#if __cplusplus
extern "C"{
#endif
#endif /* __cplusplus */


#define MAX_DATA_LEN (1024*1024 - 16) //the max data length for encryption / decryption is  8k one time


#define HI_ERR_CIPHER(fmt...)  printf(fmt)
#define HI_INFO_CIPHER(fmt...) printf(fmt)


/******************************* API Declaration *****************************/
/** \addtogroup      CIPHER */
/** @{ */  /** <!-- [CIPHER] */
/* ---CIPHER---*/
/** 
\brief  Init the cipher device.  CNcomment:ʼCIPHER豸 CNend
\attention \n
This API is used to start the cipher device.
CNcomment:ô˽ӿڳʼCIPHER豸 CNend
\param N/A                   													CNcomment: CNend
\retval ::HI_SUCCESS  Call this API succussful. 								CNcomment:APIϵͳóɹ CNend
\retval ::HI_FAILURE  Call this API fails.                                   	CNcomment:APIϵͳʧ CNend
\retval ::HI_ERR_CIPHER_FAILED_INIT  The cipher device fails to be initialized. CNcomment:CIPHER豸ʼʧ CNend
\see \n
N/A
*/
HI_S32 Cipher_Init(HI_VOID);

/** 
\brief  Deinit the cipher device.
CNcomment:\brief  ȥʼCIPHER豸 CNend
\attention \n
This API is used to stop the cipher device. If this API is called repeatedly, HI_SUCCESS is returned, but only the first operation takes effect.
CNcomment:ô˽ӿڹرCIPHER豸ظرշسɹһá CNend

\param N/A                   													CNcomment: CNend
\retval ::HI_SUCCESS  Call this API succussful. 						 		CNcomment:APIϵͳóɹ CNend
\retval ::HI_FAILURE  Call this API fails.                            			CNcomment:APIϵͳʧ CNend
\retval ::HI_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.  		CNcomment:CIPHER豸δʼ CNend
\see \n
N/A
*/
HI_VOID Cipher_Exit(HI_VOID);

/** 
\brief Obtain a cipher handle for encryption and decryption.
CNcommentһ·Cipher CNend

\param[in] cipher attributes                                             		CNcomment:cipher ԡ CNend
\param[out] phCipher Cipher handle                                       		CNcomment:CIPHER CNend
\retval ::HI_SUCCESS Call this API succussful. 						 			CNcomment:APIϵͳóɹ CNend
\retval ::HI_FAILURE Call this API fails.										CNcomment: APIϵͳʧ CNend
\retval ::HI_ERR_CIPHER_NOT_INIT  The cipher device is not initialized.  		CNcomment:CIPHER豸δʼ CNend
\retval ::HI_ERR_CIPHER_INVALID_POINT  The pointer is null.              		CNcomment:ָΪ CNend
\retval ::HI_ERR_CIPHER_FAILED_GETHANDLE  The cipher handle fails to be obtained, because there are no available cipher handles. CNcomment: ȡCIPHERʧܣûпеCIPHER CNend
\see \n
N/A
*/
HI_S32 Cipher_CreateHandle(HI_HANDLE* phCipher);

/** 
\brief Destroy the existing cipher handle. CNcomment:ѴڵCIPHER CNend
\attention \n
This API is used to destroy existing cipher handles.
CNcomment:ô˽ӿѾCIPHER CNend

\param[in] hCipher Cipher handle                 								CNcomment:CIPHER CNend
\retval ::HI_SUCCESS  Call this API succussful. 						 		CNcomment:APIϵͳóɹ CNend
\retval ::HI_FAILURE  Call this API fails.    									CNcomment:APIϵͳʧ CNend
\retval ::HI_ERR_CIPHER_NOT_INIT  The cipher device is not initialized. 		CNcomment:CIPHER豸δʼ CNend
\see \n
N/A
*/
HI_S32 Cipher_DestroyHandle(HI_HANDLE hCipher);

/** 
\brief Configures the cipher control information.
CNcomment:\brief CIPHERϢ CNend
\attention \n
Before encryption or decryption, you must call this API to configure the cipher control information.
The first 64-bit data and the last 64-bit data should not be the same when using TDES algorithm.
CNcomment:мܽǰʹô˽ӿCIPHERĿϢʹTDES㷨ʱԿǰ64 bitݲͬ CNend

\param[in] hCipher Cipher handle.                                        		CNcomment:CIPHER CNend
\param[in] pstCtrl Cipher control information.                           		CNcomment:CIPHERϢ CNend
\retval ::HI_SUCCESS Call this API succussful. 						 			CNcomment:APIϵͳóɹ CNend
\retval ::HI_FAILURE Call this API fails.                           			CNcomment:APIϵͳʧ CNend
\retval ::HI_ERR_CIPHER_NOT_INIT  The cipher device is not initialized. 		CNcomment:CIPHER豸δʼ CNend
\retval ::HI_ERR_CIPHER_INVALID_POINT  The pointer is null.             		CNcomment:ָΪ CNend
\retval ::HI_ERR_CIPHER_INVALID_PARA  The parameter is invalid.         		CNcomment: CNend
\retval ::HI_ERR_CIPHER_INVALID_HANDLE  The handle is invalid.          		CNcomment:Ƿ CNend
\see \n
N/A
*/
HI_S32 Cipher_ConfigHandle(HI_HANDLE hCipher, HI_UNF_CIPHER_CTRL_S* pstCtrl);

/** 
\brief Performs encryption. 
CNcomment:\brief мܡ CNend

\attention \n
This API is used to perform encryption by using the cipher module.
The length of the encrypted data should be a multiple of 8 in TDES mode and 16 in AES mode. Besides, the length can not be bigger than 0xFFFFF.After this operation, the result will affect next operation.If you want to remove vector, you need to config IV(config pstCtrl->stChangeFlags.bit1IV with 1) by transfering HI_UNF_CIPHER_ConfigHandle.
CNcomment:ʹCIPHERмܲTDESģʽ¼ܵݳӦ8ıAESӦ16ı⣬ݳȲܳ0xFFFFFβɺ󣬴˴βһβҪҪ´μ֮ܲǰHI_UNF_CIPHER_ConfigHandleIV(ҪpstCtrl->stChangeFlags.bit1IVΪ1) CNend
\param[in] hCipher Cipher handle  												CNcomment:CIPHER CNend
\param[in] u32SrcPhyAddr Physical address of the source data            		CNcomment:Դַ CNend
\param[in] u32DestPhyAddr Physical address of the target data           		CNcomment:Ŀַ CNend
\param[in] u32ByteLength   Length of the encrypted data                 		CNcomment:ݳ CNend
\retval ::HI_SUCCESS  Call this API succussful. 						 		CNcomment:APIϵͳóɹ CNend
\retval ::HI_FAILURE  Call this API fails.                           			CNcomment:APIϵͳʧ CNend
\retval ::HI_ERR_CIPHER_NOT_INIT  The cipher device is not initialized. 		CNcomment:CIPHER豸δʼ CNend
\retval ::HI_ERR_CIPHER_INVALID_PARA  The parameter is invalid.         		CNcomment: CNend
\retval ::HI_ERR_CIPHER_INVALID_HANDLE  The handle is invalid.          		CNcomment:Ƿ CNend
\see \n
N/A
*/
HI_S32 Cipher_Encrypt(HI_HANDLE hCipher, HI_U32 u32SrcPhyAddr, HI_U32 u32DestPhyAddr, HI_U32 u32ByteLength);

/** 
\brief м
\attention \n
This API is used to perform decryption by using the cipher module.
The length of the decrypted data should be a multiple of 8 in TDES mode and 16 in AES mode. Besides, the length can not be bigger than 0xFFFFF.After this operation, the result will affect next operation.If you want to remove vector, you need to config IV(config pstCtrl->stChangeFlags.bit1IV with 1) by transfering HI_UNF_CIPHER_ConfigHandle.
CNcomment:ʹCIPHERнܲTDESģʽ½ܵݳӦ8ıAESӦ16ı⣬ݳȲܳ0xFFFFFβɺ󣬴˴βһβҪҪ´ν֮ܲǰHI_UNF_CIPHER_ConfigHandleIV(ҪpstCtrl->stChangeFlags.bit1IVΪ1) CNend
\param[in] hCipher Cipher handle.                                        		CNcomment:CIPHER CNend
\param[in] u32SrcPhyAddr Physical address of the source data.            		CNcomment:Դַ CNend
\param[in] u32DestPhyAddr Physical address of the target data.           		CNcomment:Ŀַ CNend
\param[in] u32ByteLength Length of the decrypted data                   		CNcomment:ݳ CNend
\retval ::HI_SUCCESS Call this API succussful. 						 			CNcomment:APIϵͳóɹ CNend
\retval ::HI_FAILURE Call this API fails.                           			CNcomment:APIϵͳʧ CNend
\retval ::HI_ERR_CIPHER_NOT_INIT  The cipher device is not initialized. 		CNcomment:CIPHER豸δʼ CNend
\retval ::HI_ERR_CIPHER_INVALID_PARA  The parameter is invalid.         		CNcomment: CNend
\retval ::HI_ERR_CIPHER_INVALID_HANDLE  The handle is invalid.          		CNcomment:Ƿ CNend
\see \n
N/A
*/
HI_S32 Cipher_Decrypt(HI_HANDLE hCipher, HI_U32 u32SrcPhyAddr, HI_U32 u32DestPhyAddr, HI_U32 u32ByteLength);

HI_S32 DRV_Cipher_KladEncryptKey(HI_UNF_CIPHER_KEY_SRC_E enRootKey, 
                                 HI_UNF_CIPHER_KLAD_TARGET_E enTarget,
                                 HI_U8 u8CleanKey[16],
                                 HI_U8 u8EncryptKey[16]);

/**
\brief Get the random number.
CNcomment:\brief ȡ CNend

\attention \n
This API is used to obtain the random number from the hardware.
CNcomment:ô˽ӿڻȡ CNend

\param[out] pu32RandomNumber Point to the random number.                                        CNcomment:ֵ CNend
\retval ::HI_SUCCESS  Call this API succussful.                                                 CNcomment:APIϵͳóɹ CNend
\retval ::HI_FAILURE  Call this API fails.                                                      CNcomment: APIϵͳʧܡ CNend

\see \n
N/A
*/
HI_S32 Cipher_GetRandomNumber(HI_U32 *pu32RandomNumber);


/** 
\brief Init the hash module, if other program is using the hash module, the API will return failure.
CNcomment:\brief ʼHASHģ飬ʹHASHģ飬ʧ״̬ CNend

\attention \n
N/A

\param[in] pstHashAttr: The hash calculating structure input.                                      CNcomment:ڼhashĽṹ CNend
\param[out] pHashHandle: The output hash handle.               									CNcomment:hash CNend
\retval ::HI_SUCCESS  Call this API succussful. 						 						CNcomment:APIϵͳóɹ CNend
\retval ::HI_FAILURE  Call this API fails.                       								CNcomment: APIϵͳʧ CNend

\see \n
N/A
*/
HI_S32 Cipher_HashInit(HI_UNF_CIPHER_HASH_ATTS_S *pstHashAttr, HI_HANDLE *pHashHandle);

/** 
\brief Calculate the hash, if the size of the data to be calculated is very big and the DDR ram is not enough, this API can calculate the data one block by one block.
CNcomment:\brief hashֵҪȽϴ󣬸ýӿڿʵһblockһblockļ㣬Ƚϴ£ڴ治⡣ CNend

\attention \n
N/A

\param[in] hHashHandl:  Hash handle.                                  		CNcomment:Hash CNend
\param[in] pu8InputData:  The input data buffer.                  			CNcomment:ݻ CNend
\param[in] u32InputDataLen:  The input data length, attention: the block length input must be 4bytes aligned except the last block!			CNcomment:ݵĳȡҪ ݿĳȱ4ֽڶ룬һblock޴ơ CNend
\retval ::HI_SUCCESS  Call this API succussful. 						 	CNcomment:APIϵͳóɹ CNend
\retval ::HI_FAILURE  Call this API fails.                       		CNcomment:APIϵͳʧܡ CNend

\see \n
N/A
*/
HI_S32 Cipher_HashUpdate(HI_HANDLE hHashHandle, HI_U8 *pu8InputData, HI_U32 u32InputDataLen);



/** 
\brief Get the final hash value, after calculate all of the data, call this API to get the final hash value and close the handle.If there is some reason need to interupt the calculation, this API should also be call to close the handle.
CNcomment:ȡhashֵڼеݺ󣬵ӿڻȡյhashֵýӿͬʱرhashڼУҪжϼ㣬Ҳøýӿڹرhash CNend

\attention \n
N/A

\param[in] hHashHandle:  Hash handle.                                  		CNcomment:Hash CNend
\param[out] pu8OutputHash:  The final output hash value.                   	CNcomment:hashֵ CNend

\retval ::HI_SUCCESS  Call this API succussful. 						 	CNcomment:APIϵͳóɹ CNend
\retval ::HI_FAILURE  Call this API fails.                       		CNcomment:APIϵͳʧܡ CNend

\see \n
N/A
*/
HI_S32 Cipher_HashFinal(HI_HANDLE hHashHandle, HI_U8 *pu8OutputHash);

HI_S32 Cipher_CalcRsa(CIPHER_RSA_DATA_S *pCipherRsaData);

/** @} */  /** <!-- ==== API declaration end ==== */

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */

#endif /* __DRV_CIPHER_INTF_H__ */
